"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from re import sub

from mighty_logger.basic.lib_types.entry_type import EntryType
from mighty_logger.basic.lib_types.environment_type import EnvironmentType
from mighty_logger.basic.lib_types.sorting_key_type import SortingKeyType
from mighty_logger.src.environments import LogEnvironments
from mighty_logger.src.sorting_keys import SortingKeys

class Modifier:
	def __init__(
		self,
		entries: list[str],
		environment: EnvironmentType
	) -> None:
		self.__category_order = {
			"%": 0,
			"~": 1,
			"@": 2,
			"!": 3,
			"&": 4,
			"^": 5
		}
		self.__type_order = {
			'DEBUG': 0,
			'DEBUG-PERFORMANCE': 1,
			'PERFORMANCE': 2,
			'EVENT': 3,
			'AUDIT': 4,
			'METRICS': 5,
			'USER': 6,
			'MESSAGE': 7,
			'INFO': 8,
			'NOTICE': 9,
			'WARNING': 10,
			'!ERROR': 11,
			'!!@CRITICAL': 12,
			'RESOLVED': 13,
			'UNRESOLVED': 14,
			'INITIATION': 15,
			'PROGRESS': 16,
			'ACHIEVEMENT': 17,
			'MILESTONE': 18,
			'SUCCESS': 19,
			'FAIL': 20,
			'START-TIMER': 21,
			'TIMER-MARK': 22,
			'STOP-TIMER': 23
		}
		self.__entries: list[str] = entries
		self.__env: EnvironmentType = environment

	@property
	def entries(self) -> list[str]:
		return self.__entries

	def sort(self, key: SortingKeyType) -> None:
		cleared_entry = ""
		sorting_entries = []
		count_separators = 0

		for index, entry in enumerate(self.__entries[1:]):
			match self.__env.environment_name:
				case LogEnvironments.CONSOLE.environment_name:
					cleared_entry = sub(r"\033\[.*?m", "", entry)
				case LogEnvironments.PLAIN_CONSOLE.environment_name:
					cleared_entry = entry
				case LogEnvironments.HTML.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.MARKDOWN.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.PLAIN.environment_name:
					cleared_entry = entry
			if cleared_entry.startswith("-?"):
				parts = cleared_entry.split("*")[1].split()
				sorting_entries.append([
					self.__entries.pop(index + 1 - len(sorting_entries) - count_separators),
					" ".join(parts[0:2]),
					parts[4][0],
					parts[4][1:]
				])
				continue
			if cleared_entry.startswith("--"):
				self.__entries.pop(index + 1 - len(sorting_entries) - count_separators)
				count_separators += 1
				continue

		match key:
			case SortingKeys.SORT_ON_TIME:
				sorting_entries.sort(key=lambda x: x[1])
			case SortingKeys.SORT_ON_TIME_WITH_REVERSE:
				sorting_entries.sort(key=lambda x: x[1], reverse=True)
			case SortingKeys.SORT_ON_CATEGORY:
				sorting_entries.sort(key=lambda x: self.__category_order.get(x[2], float("inf")))
			case SortingKeys.SORT_ON_TYPE:
				sorting_entries.sort(key=lambda x: self.__type_order.get(x[3], float("inf")))

		sorted_entries = [item[0] for item in sorting_entries]

		self.__entries.insert(1, "----------------------------------Ordered logs----------------------------------")
		self.__entries.insert(2, "-----------------------------Unsorted empty entries-----------------------------")
		self.__entries[2:2] = sorted_entries
		self.__entries.append("--------------------------------------------------------------------------------")

	def search(self, keyword: str, empty: bool) -> None:
		cleared_entry = ""
		searching_entries = []
		count_separators = 0

		for index, entry in enumerate(self.__entries[1:]):
			match self.__env.environment_name:
				case LogEnvironments.CONSOLE.environment_name:
					cleared_entry = sub(r"\033\[.*?m", "", entry)
				case LogEnvironments.PLAIN_CONSOLE.environment_name:
					cleared_entry = entry
				case LogEnvironments.HTML.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.MARKDOWN.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.PLAIN.environment_name:
					cleared_entry = entry
			if cleared_entry.startswith("--"):
				self.__entries.pop(index + 1 - len(searching_entries) - count_separators)
				count_separators += 1
				continue
			if cleared_entry.startswith("-?"):
				parts = cleared_entry.split(" - ")
				searching_entries.append([
					self.__entries.pop(index + 1 - len(searching_entries) - count_separators),
					" ".join(parts[1:])
				])
				continue
			if empty:
				empty = self.__entries.pop(index + 1 - len(searching_entries) - count_separators)
				searching_entries.append([empty, empty])
				continue

		searched_entries = []
		for searching_entry in searching_entries:
			if keyword in searching_entry[1]:
				searched_entries.append(searching_entry[0])

		self.__entries.append("------------------------------Systematized entries------------------------------")
		self.__entries.extend(searched_entries)
		self.__entries.append("--------------------------------------------------------------------------------")

	def select(self, entry_type: EntryType) -> None:
		cleared_entry = ""
		selected_entries = []
		count_excess = 0

		for index, entry in enumerate(self.__entries[1:]):
			match self.__env.environment_name:
				case LogEnvironments.CONSOLE.environment_name:
					cleared_entry = sub(r"\033\[.*?m", "", entry)
				case LogEnvironments.PLAIN_CONSOLE.environment_name:
					cleared_entry = entry
				case LogEnvironments.HTML.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.MARKDOWN.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.PLAIN.environment_name:
					cleared_entry = entry
			if cleared_entry.startswith("-?"):
				parts = cleared_entry.split("*")[1].split()
				if entry_type.type_category == "":
					if parts[4][1:] == entry_type.type_name:
						selected_entries.append(self.__entries.pop(index + 1 - len(selected_entries) - count_excess))
						continue
				else:
					if parts[4][0] == entry_type.type_category:
						selected_entries.append(self.__entries.pop(index + 1 - len(selected_entries) - count_excess))
						continue
			self.__entries.pop(index + 1 - len(selected_entries) - count_excess)
			count_excess += 1

		self.__entries.append("--------------------------------Selected entries--------------------------------")
		self.__entries.extend(selected_entries)
		self.__entries.append("--------------------------------------------------------------------------------")



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from mighty_logger.basic.exceptions import ColorException
from mighty_logger.src.ansi_format import GetAnsiFormat

ColorPicker = {
	# Red
	'MAROON': [128, 0, 0],
	'DARKRED': [139, 0, 0],
	'RED': [255, 0, 0],
	'LIGHTRED': [255, 51, 51],
	'FIREBRICK': [178, 34, 34],
	'CRIMSON': [220, 20, 60],
	'INDIANRED': [205, 92, 92],
	'LIGHTCORAL': [240, 158, 128],
	'SALMON': [250, 128, 114],
	'DARKSALMON': [233, 150, 122],
	'LIGHTSALMON': [255, 160, 122],
	# Pink
	'MEDIUMVIOLETRED': [199, 21, 133],
	'DEEPPINK': [255, 20, 147],
	'PALEVIOLETRED': [219, 112, 147],
	'HOTPINK': [255, 105, 180],
	'LIGHTPINK': [255, 182, 193],
	'PINK': [255, 192, 203],
	# Orange
	'ORANGERED': [255, 69, 0],
	'TOMATO': [255, 99, 71],
	'DARKORANGE': [255, 140, 0],
	'CORAL': [255, 127, 80],
	'ORANGE': [255, 165, 0],
	# Yellow
	'DARKKHAKI': [189, 183, 107],
	'GOLD': [255, 215, 0],
	'KHAKI': [240, 230, 140],
	'PEACHPUFF': [255, 218, 185],
	'YELLOW': [255, 255, 0],
	'DARKYELLOW': [255, 204, 0],
	'PALEGOLDENROD': [238, 232, 170],
	'MOCCASIN': [255, 228, 181],
	# Purple
	'INDIGO': [75, 0, 130],
	'PURPLE': [128, 0, 128],
	'DARKMAGENTA': [139, 0, 139],
	'DARKVIOLET': [148, 0, 211],
	'DARKSLATEBLUE': [82, 61, 139],
	'BLUEVIOLET': [138, 43, 226],
	'DARKORCHID': [153, 50, 204],
	'FUCHSIA': [255, 0, 255],
	'SLATEBLUE': [106, 90, 205],
	'MEDIUMSLATEBLUE': [127, 104, 238],
	'MEDIUMORCHID': [186, 85, 211],
	'MEDIUMPURPLE': [147, 112, 219],
	'ORCHID': [218, 112, 214],
	'VIOLET': [238, 130, 238],
	'PLUM': [221, 160, 221],
	'THISTLE': [216, 191, 216],
	'LAVENDER': [230, 230, 250],
	# Green
	'DARKGREEN': [0, 100, 0],
	'GREEN': [0, 128, 0],
	'DARKOLIVEGREEN': [85, 107, 47],
	'FORESTGREEN': [34, 139, 34],
	'SEAGREEN': [46, 139, 87],
	'DARKSLATEGRAY': [47, 79, 79],
	'OLIVE': [128, 128, 0],
	'OLIVEDRAB': [107, 142, 35],
	'MEDIUMSEAGREEN': [60, 179, 113],
	'LIMEGREEN': [50, 205, 50],
	'LIME': [0, 255, 0],
	'SPRINGGREEN': [0, 255, 127],
	'MEDIUMSPRINGGREEN': [0, 250, 154],
	'DARKSEAGREEN': [143, 188, 143],
	'MEDIUMAQUAMARINE': [102, 205, 170],
	'YELLOWGREEN': [154, 205, 50],
	'LAWNGREEN': [124, 252, 0],
	'CHARTREUSE': [127, 255, 0],
	'LIGHTGREEN': [144, 238, 144],
	'GREENYELLOW': [173, 255, 47],
	'PALEGREEN': [152, 251, 152],
	# Aqua
	'TEAL': [0, 128, 128],
	'DARKCYAN': [0, 139, 139],
	'LIGHTSEAGREEN': [32, 178, 170],
	'CADETBLUE': [95, 158, 160],
	'DARKTURQUOISE': [0, 206, 209],
	'MEDIUMTURQUOISE': [72, 209, 204],
	'TURQUOISE': [64, 224, 208],
	'AQUA': [0, 255, 255],
	'AQUAMARINE': [127, 255, 212],
	'SKYBLUE': [135, 206, 235],
	'LIGHTSKYBLUE': [135, 206, 250],
	'LIGHTSTEELBLUE': [176, 196, 222],
	'LIGHTBLUE': [173, 216, 230],
	'POWDERBLUE': [176, 224, 230],
	'PALETURQUOISE': [175, 238, 238],
	# Blue
	'MIDNIGHTBLUE': [25, 25, 112],
	'NAVY': [0, 0, 128],
	'DARKBLUE': [0, 0, 139],
	'MEDIUMBLUE': [0, 0, 205],
	'BLUE': [0, 0, 255],
	'ROYALBLUE': [65, 105, 225],
	'STEELBLUE': [90, 130, 180],
	'DODGERBLUE': [30, 144, 255],
	'DEEPSKYBLUE': [0, 191, 255],
	'CORNFLOWERBLUE': [100, 149, 237],
	# Brown
	'BROWN': [165, 42, 42],
	'SADDLEBROWN': [139, 69, 19],
	'SIENNA': [160, 82, 45],
	'CHOCOLATE': [210, 105, 30],
	'DARKGOLDENROD': [184, 134, 11],
	'PERU': [205, 133, 63],
	'ROSYBROWN': [188, 143, 143],
	'GOLDENROD': [218, 165, 32],
	'SANDYBROWN': [244, 164, 96],
	'TAN': [210, 180, 140],
	'BURLYWOOD': [222, 184, 135],
	'WHEAT': [245, 222, 179],
	'NAVAJOWHITE': [255, 222, 173],
	'BISQUE': [255, 228, 196],
	'BLANCHEDALMOND': [255, 235, 205],
	# White
	'WHITE': [255, 255, 255],
	'SNOW': [255, 250, 250],
	'HONEYDEW': [240, 255, 240],
	'MINTCREAM': [245, 255, 250],
	'AZURE': [240, 255, 255],
	'LIGHTCYAN': [224, 255, 255],
	'ALICEBLUE': [240, 248, 255],
	'GHOSTWHITE': [248, 248, 255],
	'WHITESMOKE': [245, 245, 245],
	'SEASHELL': [255, 245, 238],
	'BEIGE': [245, 245, 220],
	'OLDLACE': [253, 245, 230],
	'FLORALWHITE': [255, 250, 240],
	'IVORY': [255, 255, 240],
	'ANTIQUEWHITE': [250, 235, 215],
	'LINEN': [250, 240, 230],
	'LAVENDERBLUSH': [255, 240, 245],
	'MISTYROSE': [255, 228, 225],
	'PAPAYAWHIP': [255, 239, 213],
	'LIGHTGOLDENRODYELLOW': [250, 250, 210],
	'CORNSILK': [255, 248, 220],
	'LEMONCHIFFON': [255, 250, 205],
	'LIGHTYELLOW': [255, 255, 224],
	# Gray and black
	'BLACK': [0, 0, 0],
	'DARKGRAY': [64, 64, 64],
	'DIMGRAY': [105, 105, 105],
	'SLATEGRAY': [112, 128, 144],
	'GRAY': [128, 128, 128],
	'LIGHTSLATEGRAY': [119, 136, 153],
	'SILVER': [192, 192, 192],
	'LIGHTGRAY': [211, 211, 211],
	'GAINSBORO': [220, 220, 220],
}

def DecColor(color_name: str) -> list[int, int, int]:
	if color_name in ColorPicker:
		return ColorPicker[color_name]
	else:
		raise ColorException("This color is not in the dictionary")

def HexColor(color_name: str) -> str:
	if color_name in ColorPicker:
		return '{:02x}{:02x}{:02x}'.format(*ColorPicker[color_name])
	else:
		raise ColorException("This color is not in the dictionary")

def AnsiColor(color_name: str, color_ground: str) -> str:
	if color_name in ColorPicker:
		return GetAnsiFormat("color/set/{}/{};{};{}".format(color_ground, *ColorPicker[color_name]))
	else:
		raise ColorException("This color is not in the dictionary")

def Dec2Hex(dec_colors: list[int, int, int]) -> str:
	return '{:02x}{:02x}{:02x}'.format(*dec_colors)

def Dec2Ansi(dec_colors: list[int, int, int], color_ground: str) -> str:
	return GetAnsiFormat("color/set/{}/{};{};{}".format(color_ground, *dec_colors))

def Hex2Dec(hex_color: str) -> list[int, int, int]:
	return [
		int(hex_color[:2], base=16),
		int(hex_color[2:4], base=16),
		int(hex_color[4:], base=16)
	]

def Hex2Ansi(hex_color: str, color_ground: str) -> str:
	return GetAnsiFormat("color/set/{}/{};{};{}".format(
		color_ground,
		int(hex_color[:2], base=16),
		int(hex_color[2:4], base=16),
		int(hex_color[4:], base=16)
	))

def Ansi2Dec(ansi_color: str) -> list[int, int, int]:
	return [
		int(ansi_color.split(';')[2]),
		int(ansi_color.split(';')[3]),
		int(ansi_color.split(';')[4][:-1])
	]

def Ansi2Hex(ansi_color: str) -> str:
	return '{:02x}{:02x}{:02x}'.format(
		int(ansi_color.split(';')[2]),
		int(ansi_color.split(';')[3]),
		int(ansi_color.split(';')[4][:-1])
	)



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from re import sub

from mighty_logger.basic.lib_types.environment_type import EnvironmentType
from mighty_logger.src.environments import LogEnvironments

class Exporter:
	def __init__(
			self,
			entries: list[str],
			environment: EnvironmentType
	) -> None:
		self.__entries: list[str] = entries
		self.__env: EnvironmentType = environment
		self.__csv: list[dict] = []

	@property
	def entries(self) -> list[str]:
		return self.__entries

	def export_to_csv(self) -> None:
		cleared_entry = ""
		csv_entry = {}

		for entry in self.__entries[1:]:
			match self.__env.environment_name:
				case LogEnvironments.CONSOLE.environment_name:
					cleared_entry = sub(r"\033\[.*?m", "", entry)
				case LogEnvironments.PLAIN_CONSOLE.environment_name:
					cleared_entry = entry
				case LogEnvironments.HTML.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.MARKDOWN.environment_name:
					cleared_entry = sub(r"<.*?>", "", entry)
				case LogEnvironments.PLAIN.environment_name:
					cleared_entry = entry
			if cleared_entry.startswith("-?"):
				parts = cleared_entry.split("*")
				entry_data = parts[1].split()

				csv_entry["Icon"] = entry_data[2]
				try:
					csv_entry["Duration"] = parts[0].split()[1]
				except IndexError:
					csv_entry["Duration"] = " "
				csv_entry["Time"] = " ".join(entry_data[0:2])
				csv_entry["Category"] = entry_data[4][0]
				csv_entry["Type"] = entry_data[4][1:]
				csv_entry["Message"] = " ".join(entry_data[6:])

				self.__csv.append(csv_entry.copy())
				csv_entry.clear()
				continue

	def save_to_csv(self, file_name: str) -> None:
		with open(f"{file_name}.csv", "w", encoding="utf-8") as csv:
			# Write headers (first line) to file
			headers = self.__csv[0].keys()
			csv.write(",".join(headers) + "\n")

			# Write data to file
			for row in self.__csv:
				values = [str(row[key]) for key in headers]
				csv.write(",".join(values) + "\n")



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class EntryType:
	def __init__(
		self,
		*,
		type_category: str,
		type_name: str,
		time_color: tuple,
		status_color: tuple,
		type_color: tuple,
		message_color: tuple,
		background_color: tuple,
		icon: tuple
	) -> None:
		self.__type_category: str = type_category
		self.__type_name: str = type_name
		self.__time_color: tuple = time_color
		self.__status_color: tuple = status_color
		self.__type_color: tuple = type_color
		self.__message_color: tuple = message_color
		self.__background_color: tuple = background_color
		self.__icon: tuple = icon

	@property
	def type_category(self) -> str:
		return self.__type_category

	@property
	def type_name(self) -> str:
		return self.__type_name

	@property
	def time_color(self) -> tuple:
		return self.__time_color

	@property
	def status_color(self) -> tuple:
		return self.__status_color

	@property
	def type_color(self) -> tuple:
		return self.__type_color

	@property
	def message_color(self) -> tuple:
		return self.__message_color

	@property
	def background_color(self) -> tuple:
		return self.__background_color

	@property
	def icon(self) -> tuple:
		return self.__icon



"""
A package with the implementation of various data (ANSI, colors, etc.).
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from .animations import IndefiniteAnimations, DefiniteAnimations
from .ansi_format import GetAnsiFormat
from .color_picker import DecColor,\
	HexColor,\
	AnsiColor,\
	Dec2Hex,\
	Dec2Ansi,\
	Hex2Dec,\
	Hex2Ansi,\
	Ansi2Dec,\
	Ansi2Hex
from .entry_types import LoggerEntryTypes, ProcessEntryTypes, SelectionTypes, SelectionCategories
from .environments import LogEnvironments
from .sorting_keys import SortingKeys



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from mighty_logger.basic.lib_types.environment_type import EnvironmentType
from mighty_logger.basic.exceptions import ReCreationException
from mighty_logger.basic.singleton import Singleton
from mighty_logger.src.entry_types import LoggerEntryTypes, ServiceProcessEntryTypes
from mighty_logger.src.environments import LogEnvironments
from mighty_logger.mighty_logger import MightyLogger

class Logger(Singleton):
	"""
	Lightweight Logger automates work with entry types.
	"""

	def __init__(
		self,
		*,
		program_name: str = "Unknown",
		log_environment: EnvironmentType = LogEnvironments.PLAIN,
		console_width: int = 60,
		icon_set: int = 1,
		global_bold_font: bool = False,
		global_italic_font: bool = False,
		global_invert_font: bool = False,
		global_background: bool = False
	) -> None:

		if not hasattr(self, "_Logger__logger"):
			if MightyLogger._instance is not None:
				self.__logger = MightyLogger._instance
				self.notice("An existing logger was taken into use")
			else:
				self.__logger = MightyLogger(
					program_name=program_name,
					log_environment=log_environment,
					console_width=console_width,
					icon_set=icon_set,
					global_bold_font=global_bold_font,
					global_italic_font=global_italic_font,
					global_invert_font=global_invert_font,
					global_background=global_background
				)
		else:
			raise ReCreationException("Logger class object already created")

	@property
	def might(self) -> MightyLogger:
		return self.__logger

	def debug(self, message_text: str) -> None:
		"""
		Debugging information logging:
		Can be used to log entry any information while debugging an application.

		:param message_text: Log entry message
		"""
		self.__logger.entry(LoggerEntryTypes.debug, message_text)

	def debug_performance(self, message_text: str) -> None:
		"""
		Performance debugging information logging:
		Can be used to log entry the execution time of operations or other
		performance information while the application is being debugged.
		"""
		self.__logger.entry(LoggerEntryTypes.debug_performance, message_text)

	def performance(self, message_text: str) -> None:
		"""
		Performance information logging:
		Can be used to log entry the execution time of operations or
		other application performance information.
		"""
		self.__logger.entry(LoggerEntryTypes.performance, message_text)

	def event(self, message_text: str) -> None:
		"""
		Event information logging:
		Can be used to log entry specific events in the application,
		such as button presses or mouse cursor movements.
		"""
		self.__logger.entry(LoggerEntryTypes.event, message_text)

	def audit(self, message_text: str) -> None:
		"""
		Audit information logging:
		Can be used to log entry changes in the system, such as creating or
		deleting users, as well as changes in security settings.
		"""
		self.__logger.entry(LoggerEntryTypes.audit, message_text)

	def metrics(self, message_text: str) -> None:
		"""
		Metrics information logging:
		Can be used to log entry metrics to track application performance and identify issues.
		"""
		self.__logger.entry(LoggerEntryTypes.metrics, message_text)

	def user(self, message_text: str) -> None:
		"""
		User information logging:
		Can be used to log entry custom logs to store additional information
		that may be useful for diagnosing problems.
		"""
		self.__logger.entry(LoggerEntryTypes.user, message_text)

	def message(self, message_text: str) -> None:
		"""
		Message information logging:
		Can be used for the usual output of ordinary messages about the program's operation.
		"""
		self.__logger.entry(LoggerEntryTypes.message, message_text)

	def info(self, message_text: str) -> None:
		"""
		Default information logging:
		Can be used to log entry messages with specific content about the operation of the program.
		"""
		self.__logger.entry(LoggerEntryTypes.info, message_text)

	def notice(self, message_text: str) -> None:
		"""
		Notice information logging:
		Can be used to flag important events that might be missed with a normal logging level.
		"""
		self.__logger.entry(LoggerEntryTypes.notice, message_text)

	def warning(self, message_text: str) -> None:
		"""
		Warning information logging:
		Can be used to log entry warnings that the program may work with unpredictable results.
		"""
		self.__logger.entry(LoggerEntryTypes.warning, message_text)

	def error(self, message_text: str) -> None:
		"""
		Error information logging:
		Used to log entry errors and crashes in the program.
		"""
		self.__logger.entry(LoggerEntryTypes.error, message_text)

	def critical(self, message_text: str) -> None:
		"""
		Critical error information logging:
		Used to log entry for critical and unpredictable program failures.
		"""
		self.__logger.entry(LoggerEntryTypes.critical, message_text)

	def success(self, message_text: str) -> None:
		"""
		Success information logging:
		Used to log entry a message about the success of the process.
		"""
		self.__logger.entry(ServiceProcessEntryTypes.success, message_text)

	def fail(self, message_text: str) -> None:
		"""
		Fail information logging:
		Used to log entry a message about the failed execution of the process.
		"""
		self.__logger.entry(ServiceProcessEntryTypes.fail, message_text)



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

AnsiFormat: dict = {
	'reset': {
		'on': '\033[0m',
	},

	'bold': {
		'on': '\033[1m',
		'off (doubly underlined)': '\033[21m',
	},

	'faint': {
		'on': '\033[2m',
		'off': '\033[22m',
	},

	'italic': {
		'on': '\033[3m',
		'fraktur': '\033[20m',
		'off': '\033[23m',
	},

	'underline': {
		'on': '\033[4m',
		'off': '\033[24m',
	},

	'blink': {  # not work
		'slow': '\033[5m',
		'rapid': '\033[6m',
		'off': '\033[25m',
	},

	'proportional spacing': {  # not work
		'on': '\033[26m',
		'off': '\033[50m',
	},

	'invert': {
		'on': '\033[7m',
		'off': '\033[27m',
	},

	'hide': {
		'on': '\033[8m',
		'off': '\033[28m',
	},

	'strike': {
		'on': '\033[9m',
		'off': '\033[29m',
	},

	'over line': {
		'on': '\033[53m',
		'off': '\033[55m',
	},

	'framed': {
		'on': '\033[51m',
		'encircled': '\033[52m',
		'off': '\033[54m',
	},

	'font': {
		'primary': '\033[10m',
		'1st alternative': '\033[11m',
		'2nd alternative': '\033[12m',
		'3rd alternative': '\033[13m',
		'4th alternative': '\033[14m',
		'5th alternative': '\033[15m',
		'6th alternative': '\033[16m',
		'7th alternative': '\033[17m',
		'8th alternative': '\033[18m',
		'9th alternative': '\033[19m',
	},

	'color': {
		'foreground': {
			'black': '\033[30m',
			'red': '\033[31m',
			'green': '\033[32m',
			'yellow': '\033[33m',
			'blue': '\033[34m',
			'magenta': '\033[35m',
			'cyan': '\033[36m',
			'white': '\033[37m',
		},
		'background': {
			'black': '\033[40m',
			'red': '\033[41m',
			'green': '\033[42m',
			'yellow': '\033[43m',
			'blue': '\033[44m',
			'magenta': '\033[45m',
			'cyan': '\033[46m',
			'white': '\033[47m',
		},
		'bright foreground': {
			'black': '\033[90m',
			'red': '\033[91m',
			'green': '\033[92m',
			'yellow': '\033[93m',
			'blue': '\033[94m',
			'magenta': '\033[95m',
			'cyan': '\033[96m',
			'white': '\033[97m',
		},
		'bright background': {
			'black': '\033[100m',
			'red': '\033[101m',
			'green': '\033[102m',
			'yellow': '\033[103m',
			'blue': '\033[104m',
			'magenta': '\033[105m',
			'cyan': '\033[106m',
			'white': '\033[107m',
		},
		'set': {
			'foreground': '\033[38;2;$m',
			'background': '\033[48;2;$m',
			'bright foreground': '\033[98;2;$m',
			'bright background': '\033[108;2;$m',
			'underline': '\033[58;2;$m',
		},
		'default': {
			'foreground': '\033[39m',
			'background': '\033[49m',
			'bright foreground': '\033[99m',
			'bright background': '\033[109m',
			'underline': '\033[59m',
		},
	},
}

def _RecursiveGetAnsiFormat(ansi_address: str, ansi: dict) -> str:
	split_address = ansi_address.split("/")
	# print(split_address)
	if type(ansi[split_address[0]]) == dict:
		return _RecursiveGetAnsiFormat("/".join(split_address[1:]), ansi[split_address[0]])
	else:
		if len(split_address) == 2:
			return ansi[split_address[0]].replace('$', split_address[1])
		else:
			return ansi[split_address[0]]

def GetAnsiFormat(ansi_address: str) -> str:
	return _RecursiveGetAnsiFormat(ansi_address, AnsiFormat)



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from threading import Thread
from time import sleep
from datetime import datetime

from mighty_logger.basic.lib_types.animation_type import BasicAnimationType,\
	IndefiniteAnimationType,\
	DefiniteAnimationType
from mighty_logger.basic.lib_types.entry_type import EntryType
from mighty_logger.basic.lib_types.environment_type import EnvironmentType
from mighty_logger.basic.lib_types.sorting_key_type import SortingKeyType
from mighty_logger.basic.lib_types.text_buffer_type import TextBufferType
from mighty_logger.basic.basic_logger import BasicLogger
from mighty_logger.basic.exceptions import ReCreationException, InitException
from mighty_logger.basic.exporter import Exporter
from mighty_logger.basic.modifier import Modifier
from mighty_logger.basic.text_buffer import BasicTextBuffer, TextBuffer
from mighty_logger.src.animations import IndefiniteAnimations, DefiniteAnimations
from mighty_logger.src.entry_types import ServiceLogger,\
	LoggerEntryTypes,\
	ServiceProcessEntryTypes,\
	ServiceTimerEntryTypes
from mighty_logger.src.environments import LogEnvironments
from mighty_logger.src.sorting_keys import SortingKeys

class MightyLogger(BasicLogger):
	def __init__(
		self,
		*,
		program_name: str,
		log_environment: EnvironmentType,
		console_width: int = 60,
		icon_set: int = 1,
		global_bold_font: bool = False,
		global_italic_font: bool = False,
		global_invert_font: bool = False,
		global_background: bool = False
	) -> None:
		if not hasattr(self, "_environment"):
			super().__init__(program_name, log_environment)
			self._animation: BasicAnimationType = BasicAnimationType([])
			self._icon_set = icon_set if 0 < icon_set < 5 else 1
			self._settings["global_bold_font"] = global_bold_font
			self._settings["global_italic_font"] = global_italic_font
			self._settings["global_invert_font"] = global_invert_font
			self._settings["global_background"] = global_background
			self._progress_rise = 0
			self._progress_start: datetime | None = None
			self._progress_time: str = "        "
			self._progress_interrupt = False
			self._start_timer_value: datetime | None = None
			self._buffer: TextBufferType = TextBufferType(log_environment)
			self._init_buffer(console_width)
			self._initial_log()
		else:
			raise ReCreationException("Logger class object already created")

	# ######################################################################################## #
	#                                                                                          #
	#                                         Property                                         #
	#                                                                                          #
	# ######################################################################################## #

	@property
	def settings(self) -> dict:
		return self._settings

	@property
	def buffer(self) -> list:
		return self._buffer.text_buffer

	# ######################################################################################## #
	#                                                                                          #
	#                                         Settings                                         #
	#                                                                                          #
	# ######################################################################################## #

	def _init_buffer(self, console_width: int) -> None:
		buffer_classes = {
			LogEnvironments.CONSOLE.environment_name: (TextBuffer, (self._environment, console_width)),
			LogEnvironments.PLAIN_CONSOLE.environment_name: (TextBuffer, (self._environment, console_width)),
			LogEnvironments.HTML.environment_name: (BasicTextBuffer, (self._environment,)),
			LogEnvironments.MARKDOWN.environment_name: (BasicTextBuffer, (self._environment,)),
			LogEnvironments.PLAIN.environment_name: (BasicTextBuffer, (self._environment,))
		}

		buffer_class, buffer_args = buffer_classes.get(self._environment.environment_name, (None, None))
		if buffer_class:
			if buffer_class._instance is not None:
				self._buffer = buffer_class._instance
				self.entry(
					entry_type=LoggerEntryTypes.notice,
					message_text="An existing buffer was taken into use",
					local_settings={"italic": True} if buffer_class == TextBuffer else None
				)
			else:
				self._buffer = buffer_class(*buffer_args)
		else:
			raise InitException("Text buffer initialization error")

	def _initial_log(self) -> None:
		if self._environment.weak_environment:
			self.empty("<body style='background-color: #000000; color: #ffffff;'>")
		self.empty(self._initialized_data([
			ServiceLogger.initial[0][self._environment.environment_code][self._settings["global_background"]],
			ServiceLogger.initial[1][self._environment.environment_code][self._settings["global_background"]]
		]))
		if self._environment.updatable:
			self._buffer.update_console()

	def set_icons(self, icon_set: int) -> None:
		self._icon_set = icon_set if 0 < icon_set < 5 else 1

	def set_settings(
		self,
		*,
		global_bold_font: bool = None,
		global_italic_font: bool = None,
		global_invert_font: bool = None,
		global_background: bool = None
	) -> None:
		if global_bold_font is not None:
			self._settings['global_bold_font'] = global_bold_font
		if global_italic_font is not None:
			self._settings['global_italic_font'] = global_italic_font
		if global_invert_font is not None:
			self._settings['global_invert_font'] = global_invert_font
		if global_background is not None:
			self._settings['global_background'] = global_background

	# ######################################################################################## #
	#                                                                                          #
	#                                    Inputter of Logger                                    #
	#                                                                                          #
	# ######################################################################################## #

	def empty(self, entry: str) -> None:
		self._buffer.append(entry)
		if self._environment.updatable:
			self._buffer.update_console()

	def addy(self, number_string: int, message: str) -> None:
		self._buffer.insert(number_string, message)
		if self._environment.updatable:
			self._buffer.update_console()

	def modify(self, number_string: int, message: str) -> None:
		self._buffer.replace(number_string, message)
		if self._environment.updatable:
			self._buffer.update_console()

	def catchy(self, number_string: int) -> str:
		data = self._buffer.pop(number_string)
		if self._environment.updatable:
			self._buffer.update_console()
		return data

	def extractly(self, number_string: int) -> None:
		self._buffer.remove(number_string)
		if self._environment.updatable:
			self._buffer.update_console()

	def clearly(self) -> None:
		self._buffer.clear()
		if self._environment.updatable:
			self._buffer.update_console()

	def savy(self, name_file: str, clean: bool) -> None:
		self._buffer.save(name_file, clean)
		if self._environment.updatable:
			self._buffer.update_console()

	def loady(self, name_file: str) -> None:
		self._buffer.load(name_file)
		if self._environment.updatable:
			self._buffer.update_console()

	def getty(self, input_text: str) -> str:
		self.empty(".")
		if self._environment.updatable:
			self._buffer.update_console()
		data = self._buffer.input(input_text)
		self._buffer.replace(-1, f"{input_text}{data}")
		return data

	# ######################################################################################## #
	#                                                                                          #
	#                                    Modifier of Logger                                    #
	#                                                                                          #
	# ######################################################################################## #

	def sort(self, key: SortingKeyType) -> None:
		sorter = Modifier(self._buffer.text_buffer.copy(), self._environment)
		sorter.sort(key)
		sorted_buffer = sorter.entries
		self.clearly()
		for entry in sorted_buffer:
			self.empty(entry)

	def sort_with_save(self, key: SortingKeyType, sort_file_name: str) -> None:
		original = self._buffer.text_buffer.copy()
		sorter = Modifier(self._buffer.get_data(), self._environment)
		sorter.sort(key)
		self._buffer.save(sort_file_name, False)
		self._buffer.get_data().clear()
		self._buffer.get_data().extend(original)

	def search(self, keyword: str, empty: bool) -> None:
		searcher = Modifier(self._buffer.text_buffer.copy(), self._environment)
		searcher.search(keyword, empty)
		searched_buffer = searcher.entries
		self.clearly()
		for entry in searched_buffer:
			self.empty(entry)

	def search_with_save(self, keyword: str, empty: bool, search_file_name: str) -> None:
		original = self._buffer.text_buffer.copy()
		searcher = Modifier(self._buffer.get_data(), self._environment)
		searcher.search(keyword, empty)
		self._buffer.save(search_file_name, False)
		self._buffer.get_data().clear()
		self._buffer.get_data().extend(original)

	def select(self, entry_type: EntryType) -> None:
		selector = Modifier(self._buffer.text_buffer.copy(), self._environment)
		selector.select(entry_type)
		selected_buffer = selector.entries
		self.clearly()
		for entry in selected_buffer:
			self.empty(entry)

	def select_with_save(self, entry_type: EntryType, select_file_name: str) -> None:
		original = self._buffer.text_buffer.copy()
		selector = Modifier(self._buffer.get_data(), self._environment)
		selector.select(entry_type)
		self._buffer.save(select_file_name, False)
		self._buffer.get_data().clear()
		self._buffer.get_data().extend(original)

	def export_to_csv(self, export_file_name: str) -> None:
		exporter = Exporter(self._buffer.text_buffer, self._environment)
		exporter.export_to_csv()
		exporter.save_to_csv(export_file_name)

	# ######################################################################################## #
	#                                                                                          #
	#                                        Publishers                                        #
	#                                                                                          #
	# ######################################################################################## #

	def publish_id(self) -> None:
		self.empty(f"Logger ID is {str(self._ID)}")

	def publish_program_name(self) -> None:
		self.empty(f"Program name which logging is {self._program_name}")

	def publish_environment(self) -> None:
		self.empty(f"Logger environment is {self._environment.environment_name}")

	def publish_global_settings(self) -> None:
		self.empty("Global settings:")
		self.empty(f"    Bold font is set to {str(self._settings['global_bold_font'])};")
		self.empty(f"    Italic font is set to {str(self._settings['global_italic_font'])};")
		self.empty(f"    Invert font is set to {str(self._settings['global_invert_font'])};")
		self.empty(f"    Background is set to {str(self._settings['global_background'])};")

	def publish_author(self) -> None:
		self.empty("Developed by Kalynovsky Valentin © 2023")

	def publish_license(self) -> None:
		self.empty("LICENSE")
		self.empty("    Copyright © 2023 Kalynovsky Valentin. All rights reserved.")
		self.empty("    Licensed under the Apache License, Version 2.0 (the 'License');")
		self.empty("    you may not use this file except in compliance with the License.")
		self.empty("    You may obtain a copy of the License at")
		self.empty("        http://www.apache.org/licenses/LICENSE-2.0")
		self.empty("    Unless required by applicable law or agreed to in writing, software")
		self.empty("    distributed under the License is distributed on an 'AS IS' BASIS,")
		self.empty("    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.")
		self.empty("    See the License for the specific language governing permissions and")
		self.empty("    limitations under the License.")

	def separator(self) -> None:
		self.empty(f"{'-' * 80}")

	# ######################################################################################## #
	#                                                                                          #
	#                                    Entering to Logger                                    #
	#                                                                                          #
	# ######################################################################################## #

	def entry(
		self,
		entry_type: EntryType,
		message_text: str,
		local_settings: dict = None
	) -> None:
		if local_settings is None:
			local_settings = {}
		self.empty(
			self._assemble_entry(
				entry_type,
				self._icon_set,
				self._progress_time,
				message_text,
				local_settings
			)
		)

	# ######################################################################################## #
	#                                                                                          #
	#                                  Entering to Processes                                   #
	#                                                                                          #
	# ######################################################################################## #

	def start_indefinite_process(
		self,
		message_text: str,
		animation: IndefiniteAnimationType = IndefiniteAnimations.Line,
		local_settings: dict = None
	) -> None:
		self._animation = animation

		self._progress_start = datetime.now()
		progress_stop = datetime.now()
		self._progress_time = "&" + str(progress_stop - self._progress_start).split(".")[0]
		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(ServiceProcessEntryTypes.initiation, **args)

		thread = Thread(target=self._indefinite_progress, kwargs=args)
		thread.start()

	def _indefinite_progress(
		self,
		message_text: str,
		local_settings: dict = None
	) -> None:
		if local_settings is None:
			local_settings = {}
		animation_index = 0
		self.empty(".")
		if self._environment.updatable:
			self._buffer.update_console()
		while not self._progress_interrupt:
			animation_item = self._animation.animation[animation_index]
			self._buffer.text_buffer[-1] = self._assemble_entry(
				ServiceProcessEntryTypes.process,
				self._icon_set,
				animation_item,
				message_text,
				local_settings
			)
			animation_index = (animation_index + 1) % len(self._animation.animation)
			if self._environment.updatable:
				self._buffer.update_entry()
			sleep(0.1)
		if self._environment.updatable:
			self._buffer.update_console()

	def start_definite_process(
		self,
		message_text: str,
		progress_bar: DefiniteAnimationType = DefiniteAnimations.Line,
		local_settings: dict = None
	) -> None:
		self._animation = progress_bar

		self._progress_start = datetime.now()
		progress_stop = datetime.now()
		self._progress_time = "&" + str(progress_stop - self._progress_start).split(".")[0]
		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(ServiceProcessEntryTypes.initiation, **args)

		thread = Thread(target=self._definite_progress, kwargs=args)
		thread.start()

	def _definite_progress(
		self,
		message_text: str,
		local_settings: dict = None
	) -> None:
		if local_settings is None:
			local_settings = {}
		old_progress_rise = 0
		self.empty(".")
		if self._environment.updatable:
			self._buffer.update_console()
		while not self._progress_interrupt:
			if old_progress_rise == self._progress_rise:
				continue
			else:
				old_progress_rise = self._progress_rise
				animation_item = f"{self._animation.animation[(self._progress_rise // 15) + (2 if self._progress_rise == 100 else 1)]} - {self._progress_rise} %"
				self._buffer.text_buffer[-1] = self._assemble_entry(
					ServiceProcessEntryTypes.process,
					self._icon_set,
					animation_item,
					message_text,
					local_settings
				)
				if self._environment.updatable:
					self._buffer.update_entry()
			sleep(0.1)
		if self._environment.updatable:
			self._buffer.update_console()

	def progress_rise(self, percent: int) -> None:
		self._progress_rise = percent

	def note_process(
		self,
		entry_type: EntryType,
		message_text: str,
		local_settings: dict = None
	) -> None:
		sleep(0.001)
		last = self._buffer.pop()

		progress_stop = datetime.now()
		self._progress_time = "&" + str(progress_stop - self._progress_start).split(".")[0]
		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(entry_type, **args)

		self.empty(last)

	def stop_process(
		self,
		message_text: str,
		local_settings: dict = None
	) -> None:
		self._progress_interrupt = True
		sleep(0.11)
		self._buffer.remove()

		progress_stop = datetime.now()
		self._progress_time = "&" + str(progress_stop - self._progress_start).split(".")[0]
		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(
			ServiceProcessEntryTypes.success if self._progress_rise == 100 else ServiceProcessEntryTypes.fail,
			**args
		)

		self._progress_rise = 0
		self._progress_start = None
		self._progress_time = "        "
		self._progress_interrupt = False

	# ######################################################################################## #
	#                                                                                          #
	#                                     Entering to Timer                                    #
	#                                                                                          #
	# ######################################################################################## #

	def start_timer(
		self,
		message_text: str,
		local_settings: dict = None
	) -> None:
		self._start_timer_value = datetime.now()
		stop_timer_value = datetime.now()
		self._progress_time = "^" + str(stop_timer_value - self._start_timer_value).split(".")[0]

		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(ServiceTimerEntryTypes.start_timer, **args)

		self._progress_time = "        "

	def timer_mark(
		self,
		message_text: str,
		local_settings: dict = None
	) -> None:
		mark_timer_value = datetime.now()
		self._progress_time = "^" + str(mark_timer_value - self._start_timer_value).split(".")[0]

		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(ServiceTimerEntryTypes.timer_mark, **args)

		self._progress_time = "        "

	def stop_timer(
		self,
		message_text: str,
		local_settings: dict = None
	) -> None:
		stop_timer_value = datetime.now()
		self._progress_time = "^" + str(stop_timer_value - self._start_timer_value).split(".")[0]

		args = {'message_text': message_text}
		if local_settings is not None:
			args['local_settings'] = local_settings
		self.entry(ServiceTimerEntryTypes.stop_timer, **args)

		self._start_timer_value = None
		self._progress_time = "        "



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from re import sub
from sys import stdout

from mighty_logger.basic.lib_types.environment_type import EnvironmentType
from mighty_logger.basic.lib_types.text_buffer_type import TextBufferType
from mighty_logger.basic.exceptions import ReCreationException, EnvironmentException
from mighty_logger.basic.singleton import Singleton
from mighty_logger.src.environments import LogEnvironments

class BasicTextBuffer(Singleton, TextBufferType):
	def __init__(self, env: EnvironmentType) -> None:
		if not hasattr(self, "_text_buffer"):
			if env.environment_name in [
				LogEnvironments.CONSOLE.environment_name,
				LogEnvironments.PLAIN_CONSOLE.environment_name
			]:
				raise EnvironmentException("This environment is not supported")
			else:
				super().__init__(env)
		else:
			raise ReCreationException("BasicTextBuffer class object already created")

	def append(self, message: str) -> None:
		self._text_buffer.append(f"{message}")

	def insert(self, number_string: int, message: str) -> None:
		if number_string < len(self._text_buffer):
			self._text_buffer.insert(number_string, f"{message}")
		else:
			self._text_buffer.extend([""] * (number_string - len(self._text_buffer)))
			self.append(message)

	def replace(self, number_string: int, message: str) -> None:
		if number_string < len(self._text_buffer):
			self._text_buffer[number_string] = f"{message}"
		else:
			self._text_buffer.extend([""] * (number_string - len(self._text_buffer)))
			self.append(message)

	def pop(self, number_string: int = -1) -> str:
		last = self._text_buffer.pop(number_string)
		return last

	def remove(self, number_string: int = -1) -> None:
		self._text_buffer.pop(number_string)

	def clear(self) -> None:
		self._text_buffer.clear()

	def save(self, name_file: str, clean: bool) -> None:
		match self._environment.environment_name:
			case LogEnvironments.HTML.environment_name:
				with open(f"{name_file}.html", "w", encoding="utf-8") as text_buffer_file:
					if clean:
						text_buffer_file.write('<pre>' + self._text_buffer[0] + '\n'.join([entry for entry in self._text_buffer[1:] if sub(r"<.*?>", "", entry).startswith("-")]) + '</body></pre>')
					else:
						text_buffer_file.write('<pre>' + self._text_buffer[0] + '\n'.join(self._text_buffer[1:]) + '</body></pre>')
			case LogEnvironments.MARKDOWN.environment_name:
				with open(f"{name_file}.md", "w", encoding="utf-8") as text_buffer_file:
					if clean:
						text_buffer_file.write('<pre>' + self._text_buffer[0] + '\n'.join([entry for entry in self._text_buffer[1:] if sub(r"<.*?>", "", entry).startswith("-")]) + '</body></pre>')
					else:
						text_buffer_file.write('<pre>' + self._text_buffer[0] + '\n'.join(self._text_buffer[1:]) + '</body></pre>')
			case LogEnvironments.PLAIN.environment_name:
				with open(f"{name_file}.txt", "w", encoding="utf-8") as text_buffer_file:
					if clean:
						text_buffer_file.write('\n'.join([entry for entry in self._text_buffer if entry.startswith("-")]))
					else:
						text_buffer_file.write('\n'.join(self._text_buffer))

	def load(self, name_file: str) -> None:
		match self._environment.environment_name:
			case LogEnvironments.HTML.environment_name:
				with open(f"{name_file}.html", "r", encoding="utf-8") as text_buffer_file:
					self.clear()
					self._text_buffer = text_buffer_file.read()\
						.replace("<pre>", "")\
						.replace("</body></pre>", "")\
						.replace("><", ">\n<", 1)\
						.split("\n")
			case LogEnvironments.MARKDOWN.environment_name:
				with open(f"{name_file}.md", "r", encoding="utf-8") as text_buffer_file:
					self.clear()
					self._text_buffer = text_buffer_file.read()\
						.replace("<pre>", "")\
						.replace("</body></pre>", "")\
						.replace("><", ">\n<", 1)\
						.split("\n")
			case LogEnvironments.PLAIN.environment_name:
				with open(f"{name_file}.txt", "r", encoding="utf-8") as text_buffer_file:
					self.clear()
					self._text_buffer = text_buffer_file.read().split("\n")

	def input(self, input_text: str) -> str:
		return input(f"\r{input_text}")

	def update_console(self) -> None:
		super().update_console()

	def update_entry(self) -> None:
		super().update_console()

class TextBuffer(Singleton, TextBufferType):
	def __init__(self, env: EnvironmentType, console_width: int = 60) -> None:
		if not hasattr(self, "_text_buffer"):
			if env.environment_name in [
				LogEnvironments.HTML.environment_name,
				LogEnvironments.MARKDOWN.environment_name,
				LogEnvironments.PLAIN.environment_name
			]:
				raise EnvironmentException("This environment is not supported")
			else:
				super().__init__(env)
				self._cursor_string: int = 0
				self._buffer_size: int = 0
				self.width = console_width
		else:
			raise ReCreationException("TextBuffer class object already created")

	def append(self, message: str) -> None:
		excess_console_string = len(sub(r"\033\[.*?m", "", message)) // self.width
		self._buffer_size += 1 + excess_console_string
		self._text_buffer.append(f"{message}")

	def insert(self, number_string: int, message: str) -> None:
		if number_string > self._cursor_string:
			count_empty_strings = (number_string - len(self._text_buffer))
			self._text_buffer.extend([""] * count_empty_strings)
			self._buffer_size += count_empty_strings
			self.append(message)
		else:
			excess_console_string = len(sub(r"\033\[.*?m", "", message)) // self.width
			self._buffer_size += 1 + excess_console_string
			self._text_buffer.insert(number_string, f"{message}")

	def replace(self, number_string: int, message: str) -> None:
		if number_string > self._cursor_string:
			count_empty_strings = (number_string - len(self._text_buffer))
			self._text_buffer.extend([""] * count_empty_strings)
			self._buffer_size += count_empty_strings
			self.append(message)
		else:
			old_excess_console_strings = len(sub(r"\033\[.*?m", "", self._text_buffer[number_string])) // self.width
			new_excess_console_strings = len(sub(r"\033\[.*?m", "", message)) // self.width
			self._buffer_size += new_excess_console_strings - old_excess_console_strings
			self._text_buffer[number_string] = f"{message}"

	def pop(self, number_string: int = -1) -> str:
		excess_console_string = len(sub(r"\033\[.*?m", "", self._text_buffer[-1])) // self.width
		self._buffer_size -= 1 + excess_console_string
		last = self._text_buffer.pop(number_string)
		return last

	def remove(self, number_string: int = -1) -> None:
		excess_console_string = len(sub(r"\033\[.*?m", "", self._text_buffer[-1])) // self.width
		self._buffer_size -= 1 + excess_console_string
		self._text_buffer.pop(number_string)

	def clear(self) -> None:
		if self._cursor_string == 0:
			stdout.write(f'\r\033[K')
		else:
			stdout.write(f'\033[{self._cursor_string}A\r\033[J')
		self._text_buffer.clear()
		stdout.flush()
		self._buffer_size = 0
		self._cursor_string = 0

	def save(self, name_file: str, clean: bool) -> None:
		match self._environment.environment_name:
			case LogEnvironments.CONSOLE.environment_name:
				with open(f"{name_file}.contxt", "w", encoding="utf-8") as text_buffer_file:
					if clean:
						text_buffer_file.write('\n'.join([entry for entry in self._text_buffer if sub(r"\033\[.*?m", "", entry).startswith("-")]))
					else:
						text_buffer_file.write('\n'.join(self._text_buffer))
			case LogEnvironments.PLAIN_CONSOLE.environment_name:
				with open(f"{name_file}.txt", "w", encoding="utf-8") as text_buffer_file:
					if clean:
						text_buffer_file.write('\n'.join([entry for entry in self._text_buffer if entry.startswith("-")]))
					else:
						text_buffer_file.write('\n'.join(self._text_buffer))

	def load(self, name_file: str) -> None:
		match self._environment.environment_name:
			case LogEnvironments.CONSOLE.environment_name:
				with open(f"{name_file}.contxt", "r", encoding="utf-8") as text_buffer_file:
					self.clear()
					data = text_buffer_file.read().split("\n")
					for entry in data:
						self.append(entry)
			case LogEnvironments.PLAIN_CONSOLE.environment_name:
				with open(f"{name_file}.txt", "r", encoding="utf-8") as text_buffer_file:
					self.clear()
					data = text_buffer_file.read().split("\n")
					for entry in data:
						self.append(entry)

	def input(self, input_text: str) -> str:
		data = input(f"\r{input_text}")
		stdout.write(f'\033[1A')
		stdout.flush()
		return data

	def update_console(self) -> None:
		if self._cursor_string == 0:
			stdout.write(f'\r\033[K')
		else:
			stdout.write(f'\033[{self._cursor_string}A\r\033[J')
		stdout.write('\n'.join(self._text_buffer))
		stdout.flush()
		self._cursor_string = self._buffer_size - 1

	def update_entry(self) -> None:
		stdout.write(f'\r')
		stdout.write(self._text_buffer[-1])
		stdout.flush()



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from mighty_logger.basic.lib_types.environment_type import EnvironmentType

class TextBufferType:
	def __init__(self, env: EnvironmentType) -> None:
		self._text_buffer: list[str] = []
		self._environment: EnvironmentType = env

	@property
	def text_buffer(self) -> list:
		return self._text_buffer

	def __lt__(self, message: str) -> None:
		self.append(message)

	def __gt__(self, number_string: int) -> str:
		return self.pop(number_string)

	def __lshift__(self, name_file: str) -> None:
		self.load(name_file)

	def __rshift__(self, name_file: str) -> None:
		self.save(name_file, False)

	def append(self, message: str) -> None:
		raise NotImplementedError("Method append() is not implemented in the base class")

	def insert(self, number_string: int, message: str) -> None:
		raise NotImplementedError("Method insert() is not implemented in the base class")

	def replace(self, number_string: int, message: str) -> None:
		raise NotImplementedError("Method replace() is not implemented in the base class")

	def pop(self, number_string: int = -1) -> str:
		raise NotImplementedError("Method pop() is not implemented in the base class")

	def remove(self, number_string: int = -1) -> None:
		raise NotImplementedError("Method remove() is not implemented in the base class")

	def clear(self) -> None:
		raise NotImplementedError("Method clear() is not implemented in the base class")

	def save(self, name_file: str, clean: bool) -> None:
		raise NotImplementedError("Method save() is not implemented in the base class")

	def load(self, name_file: str) -> None:
		raise NotImplementedError("Method load() is not implemented in the base class")

	def input(self, input_text: str) -> str:
		raise NotImplementedError("Method input() is not implemented in the base class")

	def update_console(self) -> None:
		raise NotImplementedError("Method update_console() is not implemented in the base class")

	def update_entry(self) -> None:
		raise NotImplementedError("Method update_entry() is not implemented in the base class")



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from random import randint
import platform
from os import getlogin
from datetime import datetime

from mighty_logger.basic.lib_types.entry_type import EntryType
from mighty_logger.basic.lib_types.environment_type import EnvironmentType
from mighty_logger.basic.exceptions import MessageException
from mighty_logger.basic.singleton import Singleton
from mighty_logger.src.environments import LogEnvironments
from mighty_logger.src.ansi_format import GetAnsiFormat

class BasicLogger(Singleton):
	def __init__(
		self,
		program_name: str,
		env: EnvironmentType
	) -> None:
		self._ID: int = randint(1000000, 9999999)
		self._program_name: str = program_name
		self._environment: EnvironmentType = env
		self._settings: dict = {}

	def _initialized_data(
		self,
		colors: list[str, str]
	) -> str:
		match self._environment.environment_name:
			case LogEnvironments.CONSOLE.environment_name:
				return (
					f"{colors[1]}" +
					f"{colors[0]}-{self._program_name}?entry> " +
					f"${platform.node()}:{getlogin()}" +
					f":{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}" +
					f"{GetAnsiFormat('reset/on')}"
				)
			case LogEnvironments.PLAIN_CONSOLE.environment_name:
				return (
					f"-{self._program_name}?entry> " +
					f"${platform.node()}:{getlogin()}" +
					f":{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}" +
					f"{GetAnsiFormat('reset/on')}"
				)
			case LogEnvironments.HTML.environment_name:
				return (
					f"<span style='background-color: #{colors[1]};'>" +
					f"<span style='color: #{colors[0]};'>-{self._program_name}?entry> " +
					f"${platform.node()}:{getlogin()}" +
					f":{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}" +
					f"</span></span>"
				)
			case LogEnvironments.MARKDOWN.environment_name:
				return (
					f"<span style='background-color: #{colors[1]};'>" +
					f"<span style='color: #{colors[0]};'>-{self._program_name}?entry> " +
					f"${platform.node()}:{getlogin()}" +
					f":{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}" +
					f"</span></span>"
				)
			case LogEnvironments.PLAIN.environment_name:
				return (
					f"-{self._program_name}?entry> " +
					f"${platform.node()}:{getlogin()}" +
					f":{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}"
				)

	def _assemble_entry(
		self,
		entry_type: EntryType,
		icon_set: int,
		animation: str,
		message_text: str,
		local_settings: dict
	) -> str:
		if len(message_text) < 10:
			raise MessageException("Message is too short (less than 10 characters)")

		bold = local_settings['bold'] if 'bold' in local_settings else self._settings['global_bold_font']
		italic = local_settings['italic'] if 'italic' in local_settings else self._settings['global_italic_font']
		invert = local_settings['invert'] if 'invert' in local_settings else self._settings['global_invert_font']
		background = local_settings['background'] if 'background' in local_settings else self._settings['global_background']

		match self._environment.environment_name:
			case LogEnvironments.CONSOLE.environment_name:
				return (
					(f"{GetAnsiFormat('bold/on')}" if bold else "") +
					(f"{GetAnsiFormat('italic/on')}" if italic else "") +
					(f"{GetAnsiFormat('invert/on')}" if invert else "") +
					f"{entry_type.background_color[self._environment.environment_code][background]}" +
					f"{entry_type.message_color[self._environment.environment_code][background]}-?entry> {animation} " +
					f"{entry_type.time_color[self._environment.environment_code][background]}*{datetime.now()} " +
					f"{entry_type.icon[icon_set]} " +
					f"{entry_type.status_color[self._environment.environment_code][background]}#STATUS: " +
					f"{entry_type.type_color[self._environment.environment_code][background]}{entry_type.type_category}{entry_type.type_name} - " +
					f"{entry_type.message_color[self._environment.environment_code][background]}{message_text}" +
					f"{GetAnsiFormat('reset/on')}"
				)
			case LogEnvironments.PLAIN_CONSOLE.environment_name:
				return (
					f"-?entry> {animation} " +
					f"*{datetime.now()} " +
					f"{entry_type.icon[icon_set]} " +
					f"#STATUS: " +
					f"{entry_type.type_category}{entry_type.type_name} - " +
					f"{message_text}"
				)
			case LogEnvironments.HTML.environment_name:
				return (
					(f"<b>" if bold else "") +
					(f"<i>" if italic else "") +
					f"<span style='background-color: #{entry_type.background_color[self._environment.environment_code][background]};'>" +
					f"<span style='color: #{entry_type.message_color[self._environment.environment_code][background]};'>-?entry> {animation} </span>" +
					f"<span style='color: #{entry_type.time_color[self._environment.environment_code][background]};'>*{datetime.now()} </span>" +
					f"{entry_type.icon[icon_set]} " +
					f"<span style='color: #{entry_type.status_color[self._environment.environment_code][background]};'>#STATUS: </span>" +
					f"<span style='color: #{entry_type.type_color[self._environment.environment_code][background]};'>{entry_type.type_category}{entry_type.type_name} - </span>" +
					f"<span style='color: #{entry_type.message_color[self._environment.environment_code][background]};'>{message_text}</span></span>" +
					(f"</i>" if italic else "") +
					(f"</b>" if bold else "")
				)
			case LogEnvironments.MARKDOWN.environment_name:
				return (
					(f"<b>" if bold else "") +
					(f"<i>" if italic else "") +
					f"<span style='background-color: #{entry_type.background_color[self._environment.environment_code][background]};'>" +
					f"<span style='color: #{entry_type.message_color[self._environment.environment_code][background]};'>-?entry> {animation} </span>" +
					f"<span style='color: #{entry_type.time_color[self._environment.environment_code][background]};'>*{datetime.now()} </span>" +
					f"{entry_type.icon[icon_set]} " +
					f"<span style='color: #{entry_type.status_color[self._environment.environment_code][background]};'>#STATUS: </span>" +
					f"<span style='color: #{entry_type.type_color[self._environment.environment_code][background]};'>{entry_type.type_category}{entry_type.type_name} - </span>" +
					f"<span style='color: #{entry_type.message_color[self._environment.environment_code][background]};'>{message_text}</span></span>" +
					(f"</i>" if italic else "") +
					(f"</b>" if bold else "")
				)
			case LogEnvironments.PLAIN.environment_name:
				return (
					f"-?entry> {animation} " +
					f"*{datetime.now()} " +
					f"{entry_type.icon[icon_set]} " +
					f"#STATUS: " +
					f"{entry_type.type_category}{entry_type.type_name} - " +
					f"{message_text}"
				)



"""
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from mighty_logger.basic.lib_types.entry_type import EntryType
from mighty_logger.src.color_picker import AnsiColor, HexColor

class ServiceLogger:
	initial = (
		(
			(AnsiColor('GOLD', "foreground"), AnsiColor('INDIGO', "foreground")),
			("", ""),
			(HexColor('GOLD'), HexColor('INDIGO')),
			(HexColor('GOLD'), HexColor('INDIGO')),
			("", "")
		), (
			("", AnsiColor('GOLD', "background")),
			("", ""),
			("", HexColor('GOLD')),
			("", HexColor('GOLD')),
			("", "")
		)
	)

class LoggerEntryTypes:
	debug = EntryType(
		type_category="%",
		type_name="DEBUG",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('BURLYWOOD', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('BURLYWOOD'), HexColor('NAVY')),
			(HexColor('BURLYWOOD'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('TAN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('TAN'), HexColor('MIDNIGHTBLUE')),
			(HexColor('TAN'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('TAN', "background")),
			("", ""),
			("", HexColor('TAN')),
			("", HexColor('TAN')),
			("", "")
		),
		icon=('🐛', '🐞', '🚧', '🔬')
	)
	debug_performance = EntryType(
		type_category="%",
		type_name="DEBUG-PERFORMANCE",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('NAVAJOWHITE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('NAVAJOWHITE'), HexColor('NAVY')),
			(HexColor('NAVAJOWHITE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('WHEAT', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('WHEAT'), HexColor('MIDNIGHTBLUE')),
			(HexColor('WHEAT'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('WHEAT', "background")),
			("", ""),
			("", HexColor('WHEAT')),
			("", HexColor('WHEAT')),
			("", "")
		),
		icon=('⏱️', '⌛️', '🔍', '📈')
	)
	performance = EntryType(
		type_category="%",
		type_name="PERFORMANCE",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('BLANCHEDALMOND', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('BLANCHEDALMOND'), HexColor('NAVY')),
			(HexColor('BLANCHEDALMOND'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('BISQUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('BISQUE'), HexColor('MIDNIGHTBLUE')),
			(HexColor('BISQUE'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('BISQUE', "background")),
			("", ""),
			("", HexColor('BISQUE')),
			("", HexColor('BISQUE')),
			("", "")
		),
		icon=('⏱️', '🚀', '📊', '⚡️')
	)
	event = EntryType(
		type_category="~",
		type_name="EVENT",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('GREENYELLOW', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('GREENYELLOW'), HexColor('NAVY')),
			(HexColor('GREENYELLOW'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('YELLOWGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('YELLOWGREEN'), HexColor('MIDNIGHTBLUE')),
			(HexColor('YELLOWGREEN'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('YELLOWGREEN', "background")),
			("", ""),
			("", HexColor('YELLOWGREEN')),
			("", HexColor('YELLOWGREEN')),
			("", "")
		),
		icon=('🔔', '🎉', '📣', '🚨')
	)
	audit = EntryType(
		type_category="~",
		type_name="AUDIT",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('MEDIUMSPRINGGREEN', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('MEDIUMSPRINGGREEN'), HexColor('NAVY')),
			(HexColor('MEDIUMSPRINGGREEN'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('SPRINGGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('SPRINGGREEN'), HexColor('MIDNIGHTBLUE')),
			(HexColor('SPRINGGREEN'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('SPRINGGREEN', "background")),
			("", ""),
			("", HexColor('SPRINGGREEN')),
			("", HexColor('SPRINGGREEN')),
			("", "")
		),
		icon=('🔍', '🔒', '📋', '🔐')
	)
	metrics = EntryType(
		type_category="~",
		type_name="METRICS",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('PALEGREEN', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('PALEGREEN'), HexColor('NAVY')),
			(HexColor('PALEGREEN'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('LIGHTGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('LIGHTGREEN'), HexColor('MIDNIGHTBLUE')),
			(HexColor('LIGHTGREEN'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('LIGHTGREEN', "background")),
			("", ""),
			("", HexColor('LIGHTGREEN')),
			("", HexColor('LIGHTGREEN')),
			("", "")
		),
		icon=('📊', '📈', '📉', '📄')
	)
	user = EntryType(
		type_category="~",
		type_name="USER",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('CHARTREUSE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('CHARTREUSE'), HexColor('NAVY')),
			(HexColor('CHARTREUSE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('LAWNGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('LAWNGREEN'), HexColor('MIDNIGHTBLUE')),
			(HexColor('LAWNGREEN'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('LAWNGREEN', "background")),
			("", ""),
			("", HexColor('LAWNGREEN')),
			("", HexColor('LAWNGREEN')),
			("", "")
		),
		icon=('👤', '👥', '🙋‍♂️', '🙋‍♀️')
	)
	message = EntryType(
		type_category="@",
		type_name="MESSAGE",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('PALETURQUOISE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('PALETURQUOISE'), HexColor('NAVY')),
			(HexColor('PALETURQUOISE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('POWDERBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('POWDERBLUE'), HexColor('MIDNIGHTBLUE')),
			(HexColor('POWDERBLUE'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('POWDERBLUE', "background")),
			("", ""),
			("", HexColor('POWDERBLUE')),
			("", HexColor('POWDERBLUE')),
			("", "")
		),
		icon=('💬', '📝', '🗒️', '📨')
	)
	info = EntryType(
		type_category="@",
		type_name="INFO",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('LIGHTSKYBLUE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('LIGHTSKYBLUE'), HexColor('NAVY')),
			(HexColor('LIGHTSKYBLUE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('SKYBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')),
			(HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('SKYBLUE', "background")),
			("", ""),
			("", HexColor('SKYBLUE')),
			("", HexColor('SKYBLUE')),
			("", "")
		),
		icon=('ℹ️', '🔍', '📌', '🔔')
	)
	notice = EntryType(
		type_category="@",
		type_name="NOTICE",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('LIGHTBLUE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('LIGHTBLUE'), HexColor('NAVY')),
			(HexColor('LIGHTBLUE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('LIGHTSTEELBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('LIGHTSTEELBLUE'), HexColor('MIDNIGHTBLUE')),
			(HexColor('LIGHTSTEELBLUE'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('LIGHTSTEELBLUE', "background")),
			("", ""),
			("", HexColor('LIGHTSTEELBLUE')),
			("", HexColor('LIGHTSTEELBLUE')),
			("", "")
		),
		icon=('📌', '📎', '🔖', '🚩')
	)
	warning = EntryType(
		type_category="!",
		type_name="WARNING",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('YELLOW', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('YELLOW'), HexColor('NAVY')),
			(HexColor('YELLOW'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKYELLOW', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('DARKYELLOW'), HexColor('MIDNIGHTBLUE')),
			(HexColor('DARKYELLOW'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKYELLOW', "background")),
			("", ""),
			("", HexColor('DARKYELLOW')),
			("", HexColor('DARKYELLOW')),
			("", "")
		),
		icon=('⚠️', '⚡️', '⛔️', '⚠️')
	)
	error = EntryType(
		type_category="!",
		type_name="!ERROR",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('PLUM', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('PLUM')),
			(HexColor('ORCHID'), HexColor('PLUM')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			("", "")
		),
		type_color=(
			(AnsiColor('FIREBRICK', "foreground"), AnsiColor('GAINSBORO', "foreground")),
			("", ""),
			(HexColor('FIREBRICK'), HexColor('GAINSBORO')),
			(HexColor('FIREBRICK'), HexColor('GAINSBORO')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKRED', "foreground"), AnsiColor('LIGHTGRAY', "foreground")),
			("", ""),
			(HexColor('DARKRED'), HexColor('LIGHTGRAY')),
			(HexColor('DARKRED'), HexColor('LIGHTGRAY')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKRED', "background")),
			("", ""),
			("", HexColor('DARKRED')),
			("", HexColor('DARKRED')),
			("", "")
		),
		icon=('❌', '🚫', '💔', '🔺')
	)
	critical = EntryType(
		type_category="!",
		type_name="!!@CRITICAL",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('PLUM', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('PLUM')),
			(HexColor('ORCHID'), HexColor('PLUM')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			("", "")
		),
		type_color=(
			(AnsiColor('FIREBRICK', "foreground"), AnsiColor('DARKSALMON', "foreground")),
			("", ""),
			(HexColor('FIREBRICK'), HexColor('DARKSALMON')),
			(HexColor('FIREBRICK'), HexColor('DARKSALMON')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKRED', "foreground"), AnsiColor('LIGHTSALMON', "foreground")),
			("", ""),
			(HexColor('DARKRED'), HexColor('LIGHTSALMON')),
			(HexColor('DARKRED'), HexColor('LIGHTSALMON')),
			("", "")
		),
		background_color=(
			("", AnsiColor('MAROON', "background")),
			("", ""),
			("", HexColor('MAROON')),
			("", HexColor('MAROON')),
			("", "")
		),
		icon=('🔥', '🚨', '⛔️', '🚒')
	)
	resolved = EntryType(
		type_category="!",
		type_name="RESOLVED",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('CHARTREUSE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			("", "")
		),
		type_color=(
			(AnsiColor('GREEN', "foreground"), AnsiColor('PALEGREEN', "foreground")),
			("", ""),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKGREEN', "foreground"), AnsiColor('LIGHTGREEN', "foreground")),
			("", ""),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKGREEN', "background")),
			("", ""),
			("", HexColor('DARKGREEN')),
			("", HexColor('DARKGREEN')),
			("", "")
		),
		icon=('✅', '❗', '🟦', '🟢')
	)
	unresolved = EntryType(
		type_category="!",
		type_name="UNRESOLVED",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			("", "")
		),
		type_color=(
			(AnsiColor('FIREBRICK', "foreground"), AnsiColor('YELLOW', "foreground")),
			("", ""),
			(HexColor('FIREBRICK'), HexColor('YELLOW')),
			(HexColor('FIREBRICK'), HexColor('YELLOW')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKRED', "foreground"), AnsiColor('DARKYELLOW', "foreground")),
			("", ""),
			(HexColor('DARKRED'), HexColor('DARKYELLOW')),
			(HexColor('DARKRED'), HexColor('DARKYELLOW')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKRED', "background")),
			("", ""),
			("", HexColor('DARKRED')),
			("", HexColor('DARKRED')),
			("", "")
		),
		icon=('❎', '❓', '🟥', '🔴')
	)

class ProcessEntryTypes:
	achievement = EntryType(
		type_category="&",
		type_name="ACHIEVEMENT",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('YELLOW', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('YELLOW'), HexColor('NAVY')),
			(HexColor('YELLOW'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKYELLOW', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('DARKYELLOW'), HexColor('MIDNIGHTBLUE')),
			(HexColor('DARKYELLOW'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKYELLOW', "background")),
			("", ""),
			("", HexColor('DARKYELLOW')),
			("", HexColor('DARKYELLOW')),
			("", "")
		),
		icon=('🏆', '🏆', '🌟', '🎖️')
	)
	milestone = EntryType(
		type_category="&",
		type_name="MILESTONE",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('CHARTREUSE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			("", "")
		),
		type_color=(
			(AnsiColor('GREEN', "foreground"), AnsiColor('PALEGREEN', "foreground")),
			("", ""),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKGREEN', "foreground"), AnsiColor('LIGHTGREEN', "foreground")),
			("", ""),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKGREEN', "background")),
			("", ""),
			("", HexColor('DARKGREEN')),
			("", HexColor('DARKGREEN')),
			("", "")
		),
		icon=('🔖', '🔖', '🎯', '🗺️')
	)

class ServiceProcessEntryTypes:
	initiation = EntryType(
		type_category="&",
		type_name="INITIATION",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('CHARTREUSE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			("", "")
		),
		type_color=(
			(AnsiColor('GREEN', "foreground"), AnsiColor('PALEGREEN', "foreground")),
			("", ""),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKGREEN', "foreground"), AnsiColor('LIGHTGREEN', "foreground")),
			("", ""),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKGREEN', "background")),
			("", ""),
			("", HexColor('DARKGREEN')),
			("", HexColor('DARKGREEN')),
			("", "")
		),
		icon=('🚀', '🚀', '🔥', '🔧')
	)
	process = EntryType(
		type_category="&",
		type_name="PROGRESS",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('PURPLE', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('PURPLE')),
			(HexColor('ORCHID'), HexColor('PURPLE')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('LIGHTSKYBLUE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('LIGHTSKYBLUE'), HexColor('NAVY')),
			(HexColor('LIGHTSKYBLUE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('SKYBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')),
			(HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('SKYBLUE', "background")),
			("", ""),
			("", HexColor('SKYBLUE')),
			("", HexColor('SKYBLUE')),
			("", "")
		),
		icon=('⏳', '🔄', '⚙️', '🕰️')
	)
	success = EntryType(
		type_category="&",
		type_name="SUCCESS",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('CHARTREUSE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			("", "")
		),
		type_color=(
			(AnsiColor('GREEN', "foreground"), AnsiColor('PALEGREEN', "foreground")),
			("", ""),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			(HexColor('GREEN'), HexColor('PALEGREEN')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKGREEN', "foreground"), AnsiColor('LIGHTGREEN', "foreground")),
			("", ""),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			(HexColor('DARKGREEN'), HexColor('LIGHTGREEN')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKGREEN', "background")),
			("", ""),
			("", HexColor('DARKGREEN')),
			("", HexColor('DARKGREEN')),
			("", "")
		),
		icon=('✔️', '🎉', '👍', '✅')
	)
	fail = EntryType(
		type_category="&",
		type_name="FAIL",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			(HexColor('ORANGE'), HexColor('ORANGE')),
			("", "")
		),
		type_color=(
			(AnsiColor('FIREBRICK', "foreground"), AnsiColor('YELLOW', "foreground")),
			("", ""),
			(HexColor('FIREBRICK'), HexColor('YELLOW')),
			(HexColor('FIREBRICK'), HexColor('YELLOW')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKRED', "foreground"), AnsiColor('DARKYELLOW', "foreground")),
			("", ""),
			(HexColor('DARKRED'), HexColor('DARKYELLOW')),
			(HexColor('DARKRED'), HexColor('DARKYELLOW')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKRED', "background")),
			("", ""),
			("", HexColor('DARKRED')),
			("", HexColor('DARKRED')),
			("", "")
		),
		icon=('❌', '🚫', '👎', '❎')
	)

class ServiceTimerEntryTypes:
	start_timer = EntryType(
		type_category="^",
		type_name="START-TIMER",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			(HexColor('ORCHID'), HexColor('LAVENDERBLUSH')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('CHARTREUSE', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			(HexColor('ORANGE'), HexColor('CHARTREUSE')),
			("", "")
		),
		type_color=(
			(AnsiColor('SEAGREEN', "foreground"), AnsiColor('PALEGREEN', "foreground")),
			("", ""),
			(HexColor('SEAGREEN'), HexColor('PALEGREEN')),
			(HexColor('SEAGREEN'), HexColor('PALEGREEN')),
			("", "")
		),
		message_color=(
			(AnsiColor('FORESTGREEN', "foreground"), AnsiColor('LIGHTGREEN', "foreground")),
			("", ""),
			(HexColor('FORESTGREEN'), HexColor('LIGHTGREEN')),
			(HexColor('FORESTGREEN'), HexColor('LIGHTGREEN')),
			("", "")
		),
		background_color=(
			("", AnsiColor('FORESTGREEN', "background")),
			("", ""),
			("", HexColor('FORESTGREEN')),
			("", HexColor('FORESTGREEN')),
			("", "")
		),
		icon=('⏰', '🕑', '🟩', '⏳')
	)
	timer_mark = EntryType(
		type_category="^",
		type_name="TIMER-MARK",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			(HexColor('ORCHID'), HexColor('DARKMAGENTA')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('KHAKI', "foreground"), AnsiColor('SIENNA', "foreground")),
			("", ""),
			(HexColor('KHAKI'), HexColor('SIENNA')),
			(HexColor('KHAKI'), HexColor('SIENNA')),
			("", "")
		),
		message_color=(
			(AnsiColor('DARKKHAKI', "foreground"), AnsiColor('SADDLEBROWN', "foreground")),
			("", ""),
			(HexColor('DARKKHAKI'), HexColor('SADDLEBROWN')),
			(HexColor('DARKKHAKI'), HexColor('SADDLEBROWN')),
			("", "")
		),
		background_color=(
			("", AnsiColor('DARKKHAKI', "background")),
			("", ""),
			("", HexColor('DARKKHAKI')),
			("", HexColor('DARKKHAKI')),
			("", "")
		),
		icon=('⌚', '🕕', '🟨', '⏱️')
	)
	stop_timer = EntryType(
		type_category="^",
		type_name="STOP-TIMER",
		time_color=(
			(AnsiColor('ORCHID', "foreground"), AnsiColor('PURPLE', "foreground")),
			("", ""),
			(HexColor('ORCHID'), HexColor('PURPLE')),
			(HexColor('ORCHID'), HexColor('PURPLE')),
			("", "")
		),
		status_color=(
			(AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")),
			("", ""),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			(HexColor('ORANGE'), HexColor('DARKRED')),
			("", "")
		),
		type_color=(
			(AnsiColor('LIGHTSKYBLUE', "foreground"), AnsiColor('NAVY', "foreground")),
			("", ""),
			(HexColor('LIGHTSKYBLUE'), HexColor('NAVY')),
			(HexColor('LIGHTSKYBLUE'), HexColor('NAVY')),
			("", "")
		),
		message_color=(
			(AnsiColor('SKYBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")),
			("", ""),
			(HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')),
			(HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')),
			("", "")
		),
		background_color=(
			("", AnsiColor('SKYBLUE', "background")),
			("", ""),
			("", HexColor('SKYBLUE')),
			("", HexColor('SKYBLUE')),
			("", "")
		),
		icon=('⏲️', '🕙', '🟪', '⌛')
	)

class SelectionTypes:
	debug = EntryType(
		type_category="",
		type_name="DEBUG",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	debug_performance = EntryType(
		type_category="",
		type_name="DEBUG-PERFORMANCE",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	performance = EntryType(
		type_category="",
		type_name="PERFORMANCE",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	event = EntryType(
		type_category="",
		type_name="EVENT",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	audit = EntryType(
		type_category="",
		type_name="AUDIT",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	metrics = EntryType(
		type_category="",
		type_name="METRICS",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	user = EntryType(
		type_category="",
		type_name="USER",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	message = EntryType(
		type_category="",
		type_name="MESSAGE",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	info = EntryType(
		type_category="",
		type_name="INFO",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	notice = EntryType(
		type_category="",
		type_name="NOTICE",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	warning = EntryType(
		type_category="",
		type_name="WARNING",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	error = EntryType(
		type_category="",
		type_name="!ERROR",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	critical = EntryType(
		type_category="",
		type_name="!!@CRITICAL",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	resolved = EntryType(
		type_category="",
		type_name="RESOLVED",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	unresolved = EntryType(
		type_category="",
		type_name="UNRESOLVED",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	achievement = EntryType(
		type_category="",
		type_name="ACHIEVEMENT",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	milestone = EntryType(
		type_category="",
		type_name="MILESTONE",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	initiation = EntryType(
		type_category="",
		type_name="INITIATION",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	process = EntryType(
		type_category="",
		type_name="PROGRESS",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	success = EntryType(
		type_category="",
		type_name="SUCCESS",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	fail = EntryType(
		type_category="",
		type_name="FAIL",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	start_timer = EntryType(
		type_category="",
		type_name="START-TIMER",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	timer_mark = EntryType(
		type_category="",
		type_name="TIMER-MARK",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	stop_timer = EntryType(
		type_category="",
		type_name="STOP-TIMER",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)

class SelectionCategories:
	debug = EntryType(
		type_category="%",
		type_name="",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	event = EntryType(
		type_category="~",
		type_name="",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	message = EntryType(
		type_category="@",
		type_name="",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	error = EntryType(
		type_category="!",
		type_name="",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	process = EntryType(
		type_category="&",
		type_name="",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)
	timer = EntryType(
		type_category="^",
		type_name="",
		time_color = (),
		status_color = (),
		type_color = (),
		message_color = (),
		background_color = (),
		icon = ()
	)



"""
A package that contains the data types that are used in the library.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from .animation_type import IndefiniteAnimationType, DefiniteAnimationType
from .entry_type import EntryType
from .environment_type import EnvironmentType
from .sorting_key_type import SortingKeyType
from .text_buffer_type import TextBufferType
